# ----------------------
# rclone.py
# by Peter Teuchert (www.coonlight.de)
# April 2021
# see www.coonlight.de/artikel/2022/2022-04-05-python-skript-fuer-cloud-backup-mit-rclone/
#
# use freely as you like, but without any guarantee!
# 
# python script for backup and restore with rclone
#
# call: rclone.py <operation> <HomeFolder> [<SourceFolderName>] [<SourceSubFolder>]
#
# this script backups and restores whole folders to an encrypted rclone remote
# these folders has to lie under a single home folder:
#
# HomeFolder
#     |
#     |--- SourceFolderName1
#     |--- SourceFolderName2
#     |--- SourceFolderName3
#     |---        ...
#
# the HomeFolder is parameter #2 for this script
# the SourceFolderNames are specified at the bottom of this file 
# in the calls of DoRClone(<SourceFolderName>), 
# or a single one is passed via parameter #3
#
# configuration of rclone via rclone config
# - unencrypted remote 
#   named <RCloneRemoteName> (here hardcoded as "MagentaCloud")
# - an encrypted remote for each SourceFolderName 
#   named <RCloneRemoteName>Crypt-<SourceFolderName> e.g. "MagentaCloudCrypt-Important"
#
# configuration file rclone.conf must exist in the folder of this script
# also the exclude file rclone-exclude-all.txt 
# and for each SourceFolderName a file rclone-exclude-<SourceFolderName>.txt
#
# don't forget to change things under "prepare" according to your needs
#
# ----------------------

import sys, os, time
import subprocess
import colorama
from colorama import Fore, Back, Style
from argparse import ArgumentParser
from datetime import datetime

colorama.init()

print(Style.BRIGHT)


# ----------------------
# prepare
# ----------------------
print(Style.BRIGHT + Fore.CYAN)

# date time string for the path for restoring the backup in
now = datetime.now()
DateTime = now.strftime("%Y-%m-%d_%H-%M-%S")
print("DateTime              = " + DateTime)

# path to the folder of this python script
ThisScriptFolder = os.path.normpath(os.path.dirname(os.path.abspath( __file__ )))
print("ThisScriptFolder      = " + ThisScriptFolder)

# path to the rclone folder with all the config files
# this path is also the root for restoring backups
RCloneFolder = os.path.normpath(ThisScriptFolder)
print("RCloneFolder          = " + RCloneFolder)

# path to the rclone executable
if(sys.platform.startswith('win')):
# in this case (Windows) in a subfolder of RCloneFolder
  RCloneExeFile = os.path.normpath(RCloneFolder + "/rclone-v1.55.0-windows-amd64/rclone.exe")
elif(sys.platform.startswith('linux')):
# for Linux just "rclone"
  RCloneExeFile = "rclone"
print("RCloneExeFile         = " + RCloneExeFile)

# name of the unencrypted rclone remote which is used by the encrypted remotes
# the encrypted remotes has to be named <RCloneRemoteName>Crypt-<SourceFolderName>
RCloneRemoteName = "MagentaCloud"
print("RCloneRemoteName      = " + RCloneRemoteName)

# path to the rclone .conf file
# in this case in the RCloneFolder itself
RCloneConfigFile = os.path.normpath(RCloneFolder + "/rclone.conf")
print("RCloneConfigFile      = " + RCloneConfigFile)

# list of the main source folders
# these folders must lie directly under the home folder (e.g. /home/peter or d:\peter)
# (the home folder is passed to this script with the second parameter)
AllSourceFolderNames = ["Privates", "Softwareentwicklung"]



# ----------------------
# parameters
# ----------------------
print(Style.BRIGHT + Fore.CYAN)

# parameters for this script:
# this script expects four parameters, the last two are optional
parser = ArgumentParser()
parser.add_argument("operation")
parser.add_argument("homefolder")
parser.add_argument("sourcefoldername", nargs="?", default="") # optional
parser.add_argument("sourcesubfolder", nargs="?", default="") # optional
args = parser.parse_args()

# no. 1: the operation to be done:
# values: list-source, size-source, backup, restore, list-remote, size-remote
Command = args.operation
print("Command               = " + Command)

# no. 2: path to the home directory, where all the source folders lie in
HomeFolder = os.path.normpath(args.homefolder)
print("HomeFolder            = " + HomeFolder)

# no. 3: just handle this one source folder
SourceFolderName = args.sourcefoldername
print("SourceFolderName      = " + SourceFolderName)

# no. 4: just handle this one source subfolder
SourceSubFolder = os.path.normpath(args.sourcesubfolder)
if(SourceSubFolder == "."):
  SourceSubFolder = ""
print("SourceSubFolder       = " + SourceSubFolder)



# ----------------------
# call function
# ----------------------
# this function is called for every single source folder
# with or without a sub folder
def DoRClone(source_folder_name, source_sub_folder = ""):
  print(Style.BRIGHT + Fore.GREEN)
  print("---------------------")
  if(source_sub_folder == ""):
    print(source_folder_name)
  else:
    print(source_folder_name + "/" + source_sub_folder)
  print("---------------------")
  
  # the path of the source folder to be backuped or restored
  SourcePath = os.path.normpath(HomeFolder + "/" + source_folder_name + "/" + source_sub_folder)
  print("SourcePath            = " + SourcePath)
  
  # the used rclone remote with subfolder
  RCloneRemote = RCloneRemoteName + "Crypt-" + source_folder_name + ":" + source_sub_folder
  print("RCloneRemote          = " + RCloneRemote)
  
  if(Command == "list-source" or Command == "size-source" or Command == "backup"):
    # the rclone exclude file for all source folders: rclone-exclude-all.txt
    RCloneExcludeFileAll = os.path.normpath(RCloneFolder + "/rclone-exclude-all.txt")
    print("RCloneExcludeFileAll  = " + RCloneExcludeFileAll)
    # the rclone exclude file for this special source folder: rclone-exclude-<source_folder_name>.txt
    RCloneExcludeFileThis = os.path.normpath(RCloneFolder + "/rclone-exclude-" + source_folder_name + ".txt")
    print("RCloneExcludeFileThis = " + RCloneExcludeFileThis)
  if(Command == "restore"):
    # restore path inside the rclone config folder 
    # namend by date and time and source folder name
    RestorePath = os.path.normpath(RCloneFolder + "/" + DateTime + "_" + source_folder_name + "/" + source_sub_folder)
    print("RestorePath           = " + RestorePath)
  print(Style.BRIGHT + Fore.WHITE)
  
  if(Command == "list-source"):
    subprocess.run([RCloneExeFile, 
                    "ls", 
                    SourcePath, 
                    "--config=" + RCloneConfigFile, 
                    "--exclude-from=" + RCloneExcludeFileAll, 
                    "--exclude-from=" + RCloneExcludeFileThis])
  elif(Command == "size-source"):
    subprocess.run([RCloneExeFile, 
                   "size", 
                   RCloneRemote, 
                    "--config=" + RCloneConfigFile, 
                    "--exclude-from=" + RCloneExcludeFileAll, 
                    "--exclude-from=" + RCloneExcludeFileThis])
  elif(Command == "backup"):
    subprocess.run([RCloneExeFile, 
                   "sync", 
                   SourcePath, 
                   RCloneRemote, 
                   "--delete-excluded", 
                   "--progress", 
                   "--stats=1000ms", 
                   "--stats-file-name-length=38", 
                   "--config=" + RCloneConfigFile, 
                   "--exclude-from=" + RCloneExcludeFileAll, 
                   "--exclude-from=" + RCloneExcludeFileThis])
  elif(Command == "restore"):
    subprocess.run([RCloneExeFile, 
                   "sync", 
                   RCloneRemote, 
                   RestorePath, 
                   "--progress", 
                   "--stats=1000ms", 
                   "--stats-file-name-length=38", 
                   "--config=" + RCloneConfigFile])
  elif(Command == "list-remote"):
    subprocess.run([RCloneExeFile, 
                    "ls", 
                    RCloneRemote, 
                    "--config=" + RCloneConfigFile])
  elif(Command == "size-remote"):
    subprocess.run([RCloneExeFile, 
                   "size", 
                   RCloneRemote, 
                   "--config=" + RCloneConfigFile])

  time.sleep(0.3)
  
  return


  
# ----------------------
# main action: call backups
# ----------------------
print(Style.BRIGHT + Fore.WHITE)

if(SourceFolderName != ""):
  # call once for the specific source folder and sub folder
  DoRClone(SourceFolderName, SourceSubFolder)
else:
  # call for all my source folders
  for source_folder_name in AllSourceFolderNames:
    DoRClone(source_folder_name)



# ----------------------
# bye
# ----------------------
print(Style.BRIGHT + Fore.GREEN)

print("all ready")

print("closing soon ...")

time.sleep(3)
